﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace WiFiPad
{
    /// <summary>
    /// This is the logic layer of the aplication
    /// </summary>
    class CarControl
    {
        private string LeftRight = "";
        private string direction ="";
        private SocketClient socketCli;
		private double speedLog;
		private int lastSpeedLog;
		private float direccioOld = 0;

		public string Host = "";
		public int Port = -1;
		public bool Debug = false;

        /// <summary>
        /// Get socket client connexion status
        /// </summary>
		public bool Connected
		{
			get{
				return (socketCli != null && socketCli.IsConnected);
			}
		}

        /// <summary>
        /// Init the socket client
        /// </summary>
        /// <param name="host">Host IP or Name</param>
        /// <param name="port">Port number</param>
        /// <returns></returns>
        public bool Init(string host, int port)
        {
            Host = host;
            Port = port;
            bool ret = false;

            if (Host.Trim() != "" && Port != -1)
            {
                try
                {
                    socketCli = new SocketClient(Host, Port); 
                    socketCli.Connect();
                    if(socketCli.Receive() =="*Ready*")
                    {
                        ret = true;
	                    if (Debug)
	                    {
	                        Console.WriteLine("Ok. Connexion open.");
	                    }
	                    System.Threading.Thread.Sleep(300);

						string str = socketCli.SendReceive("$$init\r\n");
						if (Debug)
						{
							Console.WriteLine(str);
						}
                    }
                }
                catch (Exception ex)
                {
                    Console.WriteLine("ERROR: " + ex.Message);
                }
            }
            return ret;
        }

        /// <summary>
        /// Close the connexion and disposes related objects
        /// </summary>
        /// <returns></returns>
        public bool Close()
        {
            bool ret = false;
            if (socketCli != null)
            {
                socketCli.Disconnect();
                socketCli.Dispose();
                ret = true;
            }
            return ret;
        }

        /// <summary>
        /// Set the traction motor at the specified speed
        /// </summary>
        /// <param name="speed">Signed integer for the speed value</param>
        public void Traction(int speed)
        {
            if (socketCli.IsConnected)
            {
                string str = "";
                if (speed > 0)
                    direction = "f";
                else
                    direction = "b";

                int uSpeed = Math.Abs(speed);
                speedLog = Math.Log(uSpeed + 1f, 5f) * 30f;

                if ((int)speedLog != lastSpeedLog)
                {
                    lastSpeedLog = (int)speedLog;
   
                    //str = socketCli.SendReceive("$$" + direction + ":" + Math.Abs(speedLog) + "\r\n");
					socketCli.SendAsync("$$" + direction + ":" + Math.Abs(speedLog) + "\r\n");

                    if (Debug)
                    {
                        Console.WriteLine(str+ ". Move " + direction + " :" + (int)speedLog);
                    }
                }
            }
        }

        /// <summary>
        /// Set the direcction motor at the specified power value
        /// </summary>
        /// <param name="direccio">Power level for the motor</param>
        public void Direcction(int direccio)
        {
            const int limitDireccio = 30;
            if (direccio < -limitDireccio) direccio = -limitDireccio;
            if (direccio > limitDireccio) direccio = limitDireccio;

            if (socketCli.IsConnected)
            {
                if (direccio > 0)
                    LeftRight = "l";
                else
                    LeftRight = "r";

                if ((int)direccio != (int)direccioOld) 
                {
                    direccioOld = direccio;
                    //string str = socketCli.SendReceive("$$" + LeftRight + ":" + Math.Abs(direccio) + "\r\n");
					socketCli.SendAsync("$$" + LeftRight + ":" + Math.Abs(direccio) + "\r\n"); //, socketCli);
                }
            }
        }

        /// <summary>
        /// Request the sonar distance to the ostacles
        /// </summary>
        /// <returns>Float value of the distance in centimeters</returns>
        public float Sonar()
        {
            float ret = 0.0f;
            if (socketCli.IsConnected)
            {
                try
                {
					socketCli.ClearQueue();
                    string str = socketCli.SendReceive("$$sonar\r\n");
                    ret = (float)Convert.ToDouble(str.Substring(3, str.Length - 5).Replace('.',','));
                }
                catch(Exception ex) {
                    if (Debug)
                    {
                        Console.WriteLine(ex.Message);
                    }
                }
            }
            return ret;     
        }


        /// <summary>
        /// Enable/Dissble the automatic stop function for near obstacles
        /// </summary>
        public void Obstacles()
        {
            if (socketCli.IsConnected)
            {
                try
                {
                    string str = socketCli.SendReceive("$$obstacles\r\n");
                }
                catch (Exception ex)
                {
                    if (Debug)
                    {
                        Console.WriteLine(ex.Message);
                    }
                }
            }
        }


        /// <summary>
        /// Stop traction and direction motors
        /// </summary>
        public void Stop()
        {
            string str = "";
            if (socketCli.IsConnected)
            {
                str = socketCli.SendReceive("$$stop\r\n");
            }
        }

        /// <summary>
        /// Set the power level of the status led
        /// </summary>
        /// <param name="power">Integer value of the power level</param>
        public void Led(int power)
        {
            if (socketCli.IsConnected)
            {
                socketCli.Send("$$led:"+power+"\r\n");
            }
        }
    }
}
