﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.IO;
using System.Net;
using System.Net.Sockets;
using System.Threading;
using System.Collections;

namespace WiFiPad
{
    /// <summary>
    /// Implements the connection logic for the socket client.
	/// This example is from http://www.codeproject.com/Articles/22918/How-To-Use-the-SocketAsyncEventArgs-Class
    /// </summary>
    internal sealed class SocketClient : IDisposable
    {
        /// <summary>
        /// Constants for socket operations.
        /// </summary>
        private const Int32 ReceiveOperation = 1, SendOperation = 0;

        /// <summary>
        /// The socket used to send/receive messages.
        /// </summary>
        private Socket clientSocket;

        /// <summary>
        /// Flag for connected socket.
        /// </summary>
        private Boolean connected = false;

        /// <summary>
        /// Listener endpoint.
        /// </summary>
        private IPEndPoint hostEndPoint;

        /// <summary>
        /// Signals a connection.
        /// </summary>
        private static AutoResetEvent autoConnectEvent = new AutoResetEvent(false);

        /// <summary>
        /// Signals the send/receive operation.
        /// </summary>
        private static AutoResetEvent[] autoSendReceiveEvents = new AutoResetEvent[]
        {
            new AutoResetEvent(false),
            new AutoResetEvent(false)
        };

		/// <summary>
		/// The thread for sending async commands.
		/// </summary>
		private Thread oThread = null; 

		/// <summary>
		/// The async commands sender.
		/// </summary>
		private CommandsSender cSender;


        /// <summary>
        /// Create an uninitialized client instance.  
        /// To start the send/receive processing
        /// call the Connect method followed by SendReceive method.
        /// </summary>
        /// <param name="hostName">Name of the host where the listener is running.</param>
        /// <param name="port">Number of the TCP port from the listener.</param>
        internal SocketClient(String hostName, Int32 port)
        {
            IPAddress[] addressList;

            System.Net.IPAddress ip;
            if (System.Net.IPAddress.TryParse(hostName, out ip))
            {
                addressList = new IPAddress[1];
                addressList[0] = ip;
            }
            else
            {
                // Get host related information.
                IPHostEntry host = Dns.GetHostEntry(hostName);
                //Dns.Resolve(Dns.GetHostName());

                // Addres of the host.
                addressList = host.AddressList;
            }
            // Instantiates the endpoint and socket.
            this.hostEndPoint = new IPEndPoint(addressList[addressList.Length - 1], port);
            this.clientSocket = new Socket(this.hostEndPoint.AddressFamily, SocketType.Stream, ProtocolType.Tcp);
        }

        /// <summary>
        /// Connect to the host.
        /// </summary>
        /// <returns>True if connection has succeded, else false.</returns>
        internal void Connect()
        {
            SocketAsyncEventArgs connectArgs = new SocketAsyncEventArgs();

            connectArgs.UserToken = this.clientSocket;
            connectArgs.RemoteEndPoint = this.hostEndPoint;
            connectArgs.Completed += new EventHandler<SocketAsyncEventArgs>(OnConnect);

            clientSocket.ConnectAsync(connectArgs);
            autoConnectEvent.WaitOne();

            SocketError errorCode = connectArgs.SocketError;
            if (errorCode != SocketError.Success)
            {
                throw new SocketException((Int32)errorCode);
            }
        }

        /// <summary>
        /// Disconnect from the host.
        /// </summary>
        internal void Disconnect()
        {
            clientSocket.Disconnect(false);
        }

        /// <summary>
        /// Check if is connect to a host
        /// </summary>
        internal bool IsConnected
        {
			get 
			{
				return clientSocket != null && clientSocket.Connected;
			}
        }

        private void OnConnect(object sender, SocketAsyncEventArgs e)
        {
            // Signals the end of connection.
            autoConnectEvent.Set();

            // Set the flag for socket connected.
            this.connected = (e.SocketError == SocketError.Success);
        }

        private void OnReceive(object sender, SocketAsyncEventArgs e)
        {
            // Signals the end of receive.
            autoSendReceiveEvents[SendOperation].Set();
        }

        private void OnSend(object sender, SocketAsyncEventArgs e)
        {
            // Signals the end of send.
            autoSendReceiveEvents[ReceiveOperation].Set();

            if (e.SocketError == SocketError.Success)
            {
                if (e.LastOperation == SocketAsyncOperation.Send)
                {
                    // Prepare receiving.
                    Socket s = e.UserToken as Socket;

                    byte[] receiveBuffer = new byte[255];
                    e.SetBuffer(receiveBuffer, 0, receiveBuffer.Length);
                    e.Completed += new EventHandler<SocketAsyncEventArgs>(OnReceive);
                    s.ReceiveAsync(e);
                }
            }
            else
            {
                this.ProcessError(e);
            }
        }

        /// <summary>
        /// Close socket in case of failure and throws a SockeException according to the SocketError.
        /// </summary>
        /// <param name="e">SocketAsyncEventArg associated with the failed operation.</param>
        private void ProcessError(SocketAsyncEventArgs e)
        {
            Socket s = e.UserToken as Socket;
            if (s.Connected)
            {
                // close the socket associated with the client
                try
                {
                    s.Shutdown(SocketShutdown.Both);
                }
                catch (Exception)
                {
                    // throws if client process has already closed
                }
                finally
                {
                    if (s.Connected)
                    {
                        s.Close();
                    }
                }
            }

            // Throw the SocketException
            throw new SocketException((Int32)e.SocketError);
        }

        /// <summary>
        /// Exchange a message with the host.
        /// </summary>
        /// <param name="message">Message to send.</param>
        /// <returns>Message sent by the host.</returns>
        internal String SendReceive(String message)
        {
            if (this.connected)
            {
                // Create a buffer to send.
                Byte[] sendBuffer = Encoding.ASCII.GetBytes(message);

                // Prepare arguments for send/receive operation.
                SocketAsyncEventArgs completeArgs = new SocketAsyncEventArgs();
                completeArgs.SetBuffer(sendBuffer, 0, sendBuffer.Length);
                completeArgs.UserToken = this.clientSocket;
                completeArgs.RemoteEndPoint = this.hostEndPoint;
                completeArgs.Completed += new EventHandler<SocketAsyncEventArgs>(OnSend);

                // Start sending asyncronally.
                clientSocket.SendAsync(completeArgs);

                // Wait for the send/receive completed.
                AutoResetEvent.WaitAll(autoSendReceiveEvents);

                // Return data from SocketAsyncEventArgs buffer.
                return Encoding.ASCII.GetString(completeArgs.Buffer, completeArgs.Offset, completeArgs.BytesTransferred);
            }
            else
            {
                throw new SocketException((Int32)SocketError.NotConnected);
            }
        }

        /// <summary>
        /// Receive a message from the host.
        /// </summary>
        /// <returns>Message sent by the host.</returns>
        internal String Receive()
        {
            if (this.connected)
            {
                // Create a buffer to send.
                Byte[] sendBuffer = Encoding.ASCII.GetBytes("");

                // Prepare arguments for send/receive operation.
                SocketAsyncEventArgs completeArgs = new SocketAsyncEventArgs();
                completeArgs.UserToken = this.clientSocket;

                // Prepare receiving.
                autoSendReceiveEvents[ReceiveOperation].Set();
                Socket s = completeArgs.UserToken as Socket;
                byte[] receiveBuffer = new byte[255];
                completeArgs.SetBuffer(receiveBuffer, 0, receiveBuffer.Length);
                completeArgs.Completed += new EventHandler<SocketAsyncEventArgs>(OnReceive);
                s.ReceiveAsync(completeArgs);

                // Wait for the receive completed.
                AutoResetEvent.WaitAll(autoSendReceiveEvents);

                // Return data from SocketAsyncEventArgs buffer.
                return Encoding.ASCII.GetString(completeArgs.Buffer, completeArgs.Offset, completeArgs.BytesTransferred);
            }
            else
            {
                throw new SocketException((Int32)SocketError.NotConnected);
            }
        }

        /// <summary>
        /// Sends a message to the host.
        /// Wait until the message is sent.
        /// </summary>
        /// <param name="message">Message to send.</param>
        internal void Send(String message)
        {
            if (this.connected)
            {
                // Create a buffer to send.
                Byte[] sendBuffer = Encoding.ASCII.GetBytes(message);

                // Prepare arguments for send/receive operation.
                SocketAsyncEventArgs completeArgs = new SocketAsyncEventArgs();
                completeArgs.SetBuffer(sendBuffer, 0, sendBuffer.Length);
                completeArgs.UserToken = this.clientSocket;
                completeArgs.RemoteEndPoint = this.hostEndPoint;
                completeArgs.Completed += new EventHandler<SocketAsyncEventArgs>(OnSend);

                // Start sending asyncronally.
                clientSocket.SendAsync(completeArgs);

                autoSendReceiveEvents[ReceiveOperation].Set();
                // Wait for the send completed.
                AutoResetEvent.WaitAll(autoSendReceiveEvents);
            }
            else
            {
                throw new SocketException((Int32)SocketError.NotConnected);
            }
        }

        /// <summary>
        /// Sends a message to the host.
        /// Don't wait until the message is sent.
        /// </summary>
        /// <param name="message">Message to send.</param>
        /// <returns>Message sent to the queue.</returns>
		internal bool SendAsync(String message) 
		{
			bool ret = false;
			if(oThread == null && clientSocket != null)
			{
				cSender = new CommandsSender(this);

				// Create the thread object, passing in the cSender.QueueSendSockets method
				// via a ThreadStart delegate. This does not start the thread.
				oThread =	new Thread(new ThreadStart(cSender.QueueSendSockets));
	
				// Start the thread
				oThread.Start();

				// Spin for a while waiting for the started thread to become alive:
				while (!oThread.IsAlive);
			}
			if (cSender != null && cSender.commandQueue.Count <10){
				cSender.commandQueue.Enqueue( message );
				ret = true;
			}
			return ret;
		}

        /// <summary>
        /// Clear the queue of pending messages to send.
        /// </summary>
        /// <returns></returns>
		internal bool ClearQueue() 
		{
			bool ret = false;
			if (cSender != null && cSender.commandQueue.Count >0){
				cSender.commandQueue.Clear();
				ret = true;
			}
			return ret;
		}

        #region IDisposable Members

        /// <summary>
        /// Disposes the instance of SocketClient.
        /// </summary>
        public void Dispose()
        {
            autoConnectEvent.Close();
            autoSendReceiveEvents[SendOperation].Close();
            autoSendReceiveEvents[ReceiveOperation].Close();
            if (this.clientSocket.Connected)
            {
                this.clientSocket.Close();
            }

			if (oThread != null)
			{
				oThread.Abort();
				// Wait until oThread finishes. Join also has overloads
				// that take a millisecond interval or a TimeSpan object.
				oThread.Join();
			}
        }

        #endregion
    }
}
