/*
 * qik2s9v1.c
 *
 *  Created on: 15/12/2012
 *      Author: Francec Pujol Contreras
 */

/* FreeRTOS.org includes. */
#include "FreeRTOS.h"
#include "task.h"
#include "semphr.h"
#include "type.h"

#include <stdio.h>
#include <stdarg.h>
#include <stdlib.h>
#include <string.h>

#include "uart.h"
#include "uartTools.h"
#include "qik2s9v1.h"

#define QIK_COMMAND_BUFF_LEN 				( 5 )
static uint8_t cmd[QIK_COMMAND_BUFF_LEN];
static uint32_t uart = 99999;
static xSemaphoreHandle xMutex;
const uint32_t UARTtimeOut =1;


/*****************************************************************************
** Function name:		qikReset
**
** Descriptions:		Hardware reset the qik 2s9v1 device.
** 						J6-23 (P0.21) of LPC1769 to QIK (active low) reset pin
**
** parameters:			None
** Returned value:		None
**
*****************************************************************************/
void qikReset()
{
	const portTickType xDelay1ms = 1 / portTICK_RATE_MS;
	// Initialize pin
	LPC_PINCON->PINSEL1 &= ( ~( 3 << 10 ) );
	LPC_GPIO0->FIODIR |= ( 1 << RST_QIK_PIN );

	LPC_GPIO0->FIOCLR = 1 << RST_QIK_PIN;
	vTaskDelay(xDelay1ms);
	LPC_GPIO0->FIOSET = 1 << RST_QIK_PIN;
}


/*****************************************************************************
** Function name:		qikInit
**
** Descriptions:		Defines the used uart, its baudrate and do a reset
** 						in Qik device. Then instructs Qik to exit autodetect
** 						mode and enter normal operation. This operation
** 						produce a 1 second delay.
**
** parameters:			uart, baudrate
** Returned value:		TRUE is successful
**
*****************************************************************************/
uint32_t qikInit(uint32_t useUart, uint32_t baudrate)
{
	const portTickType xDelay1s = 1000 / portTICK_RATE_MS;
	uint32_t ret = FALSE;

	if (useUart < MAX_UART_NUMBER)
	{
		uart = useUart;
		UARTInit(uart, baudrate); // init uart

		qikReset();
		vTaskDelay(xDelay1s);

		cmd[0] = QIK_INIT;  // Init baudrate: autodetect
		UARTSend(uart, cmd, 1);

		ret = TRUE;
		xMutex = xSemaphoreCreateMutex();

		// Verify that the semaphore was created successfully.
		if( ret == FALSE || xMutex == NULL )
			ret = FALSE;
	}
	return ret;
}

/*****************************************************************************
** Function name:		QikVersion
**
** Descriptions:		Get Firmware Version
**
** parameters:			char* for the return message of the module response
** Returned value:		TRUE is successful
**
*****************************************************************************/
uint32_t QikVersion(char* response)
{
	uint32_t ret = FALSE;
	uint32_t count =0;

	if (uart < MAX_UART_NUMBER && xMutex != NULL)
	{
		xSemaphoreTake( xMutex, portMAX_DELAY );
		{
			cmd[0]= QIK_GET_FIRMWARE_VERSION;
			UARTSend(uart, cmd, 1);
			cmd[0] =0;

			count = UartScanf(cmd, uart, UARTtimeOut);
			//dUartPrintfLn("Get Firmware Version. Return %i bytes: First byte: 0x%X %c", count, cmd[0], cmd[0]);
			if (count ==1)
			{
				//char value[3];
				sprintf(response, "%c", cmd[0]);
				ret = TRUE;
			}
		}
		xSemaphoreGive( xMutex );
	}
	return ret;
}

/*****************************************************************************
** Function name:		QikGetError
**
** Descriptions:		Get device error byte
	* bits 0 – 2: unused
	* bit 3: Data Overrun Error
		A hardware-level error that occurs when serial data is received while
		the hardware serial receive buffer is full. This error should not occur
		during normal operation.
	* bit 4: Frame Error
		A hardware-level error that occurs when a byte’s stop bit is
		not detected at the expected place. This error can occur if you are
		communicating at a baud rate that differs from the qik’s baud rate.
	* bit 5: CRC Error
		This error occurs when the qik is running in CRC-enabled mode (i.e.
		the CRC-enable jumper is in place) and the cyclic redundancy check (CRC)
		byte added to the end of the command packet does not match what the
		qik has computed as that packet’s CRC. In such a case, the qik will
		ignore the command packet and generate a CRC error. See Section 6 for
		more information on cyclic redundancy checking.
	* bit 6: Format Error
		This error occurs when the qik receives an incorrectly formatted or
		nonsensical command packet. For example, if the command byte does not
		match a known command, data bytes are outside of the allowed range
		for their particular command, or an unfinished command packet is
		interrupted by another command packet, a format error will be generated.
	* bit 7: Timeout
		It is possible to use a configuration parameter to enable the qik’s
		serial timeout feature (see Section 5.c). When enabled, the qik will
		generate a timeout error if the timeout period set by the configuration
		parameter elapses. The timeout timer is reset every time a valid
		command packet is received.
**
** parameters:			char* for the return message of the module response
** Returned value:		TRUE is successful
**
*****************************************************************************/
uint32_t QikGetError(char* response)
{
	uint32_t errorByte = 999999;
	uint32_t count =0;

	if (uart < MAX_UART_NUMBER && xMutex != NULL)
	{
		xSemaphoreTake( xMutex, portMAX_DELAY );
		{
			cmd[0]= QIK_GET_ERROR_BYTE;
			UARTSend(uart, cmd, 1);
			cmd[0] =0;

			count = UartScanf(cmd, uart, UARTtimeOut);
			if (count ==1)
			{
				errorByte = cmd[0];
				sprintf(response, "Error Byte Ok");
			}
		}
		xSemaphoreGive( xMutex );
	}
	return errorByte;
}

/*****************************************************************************
** Function name:		QikGetDeviceId
**
** Descriptions:		Get device id
**
** parameters:			char* for the return message of the module response
** Returned value:		TRUE is successful
**
*****************************************************************************/
uint32_t QikGetDeviceId(char* response)
{
	uint32_t deviceId = 999999;
	uint32_t count =0;

	if (uart < MAX_UART_NUMBER && xMutex != NULL)
	{
		xSemaphoreTake( xMutex, portMAX_DELAY );
		{
			cmd[0]= QIK_GET_CONFIGURATION_PARAMETER;
			cmd[1]= QIK_CONFIG_DEVICE_ID;
			UARTSend(uart, cmd, 2);
			cmd[0] =0;

			count = UartScanf(cmd, uart, UARTtimeOut);
			if (count ==1)
			{
				deviceId = cmd[0];
				sprintf(response, "Device ID Ok");
			}
		}
		xSemaphoreGive( xMutex );
	}
	return deviceId;
}

/*****************************************************************************
** Function name:		QikGetPWMparameter
**
** Descriptions:		Get device PWM Parameter. Values are:
**	0 = high-frequency, 7-bit mode (PWM frequency of 31.5 kHz, is ultrasonic)
**	1 = high-frequency, 8-bit mode (PWM frequency of 15.7 kHz)
**	2 = low-frequency,  7-bit mode (PWM frequency of 7.8 kHz)
**	3 = low-frequency,  8-bit mode (PWM frequency of 3.9 kHz)
**
**
** parameters:			char* for the return message of the module response
** Returned value:		PWM Parameter value
**
*****************************************************************************/
uint32_t QikGetPWMparameter(char* response)
{
	uint32_t PWMparameter = 999999;
	uint32_t count =0;

	if (uart < MAX_UART_NUMBER && xMutex != NULL)
	{
		xSemaphoreTake( xMutex, portMAX_DELAY );
		{
			cmd[0]= QIK_GET_CONFIGURATION_PARAMETER;
			cmd[1]= QIK_CONFIG_PWM_PARAMETER;
			UARTSend(uart, cmd, 2);
			cmd[0] =0;

			count = UartScanf(cmd, uart, UARTtimeOut);
			if (count ==1)
			{
				PWMparameter = cmd[0];
				sprintf(response, "PWM Parameter Ok");
			}
		}
		xSemaphoreGive( xMutex );
	}
	return PWMparameter;
}

/*****************************************************************************
** Function name:		QikGetShutDownOnError
**
** Descriptions:		Get device Shutdown motors on error.
** 						When this parameter has a value of 1, both motors
** 						M0 and M1 are stopped as a safety precaution whenever
**						an error occurs; otherwise, if this parameter has
**						a value of 0, errors will not affect the motors
**
**
** parameters:			char* for the return message of the module response
** Returned value:		Shutdown on error value
**
*****************************************************************************/
uint32_t QikGetShutDownOnError(char* response)
{
	uint32_t ShutDownOnError = 999999;
	uint32_t count =0;

	if (uart < MAX_UART_NUMBER && xMutex != NULL)
	{
		xSemaphoreTake( xMutex, portMAX_DELAY );
		{
			cmd[0]= QIK_GET_CONFIGURATION_PARAMETER;
			cmd[1]= QIK_CONFIG_SHUT_DOWN_MOTORS_ON_ERROR;
			UARTSend(uart, cmd, 2);
			cmd[0] =0;

			count = UartScanf(cmd, uart, UARTtimeOut);
			if (count ==1)
			{
				ShutDownOnError = cmd[0];
				sprintf(response, "Shutdown on error Parameter Ok");
			}
		}
		xSemaphoreGive( xMutex );
	}
	return ShutDownOnError;
}

/*****************************************************************************
** Function name:		QikGetSerialTimeout
**
** Descriptions:		Get device serial timeout parameter. Valid values are:
**						When this parameter has a value of 0, the serial
**						timeout feature is inactive. Otherwise, the value of
**						this parameter controls how much time can elapse
**						between receptions of valid command packets before
**						a serial timeout error is generated.
**
**
** parameters:			char* for the return message of the module response
** Returned value:		Serial timeout value
**
*****************************************************************************/
uint32_t QikGetSerialTimeout(char* response)
{
	uint32_t serialTimeout = 999999;
	uint32_t count =0;

	if (uart < MAX_UART_NUMBER && xMutex != NULL)
	{
		xSemaphoreTake( xMutex, portMAX_DELAY );
		{
			cmd[0]= QIK_GET_CONFIGURATION_PARAMETER;
			cmd[1]= QIK_CONFIG_SERIAL_TIMEOUT;
			UARTSend(uart, cmd, 2);
			cmd[0] =0;

			count = UartScanf(cmd, uart, UARTtimeOut);
			if (count ==1)
			{
				serialTimeout = cmd[0];
				sprintf(response, "Serial timeout Parameter Ok");
			}
		}
		xSemaphoreGive( xMutex );
	}
	return serialTimeout;
}

/*****************************************************************************
** Function name:		setConfigurationParameter
**
** Descriptions:		Set the configuration parameter to a specified value.
**
** parameters:			integer parameter to configure
** 						integer value to set
** 						char* for the return message of the module response
** Returned value:		TRUE is successful
**
*****************************************************************************/
uint32_t setConfigurationParameter(uint32_t parameter, uint32_t value, char* response)
{
	uint32_t res = 999999;
	uint32_t count =0;

	if (uart < MAX_UART_NUMBER && xMutex != NULL)
	{
		xSemaphoreTake( xMutex, portMAX_DELAY );
		{
			cmd[0] = QIK_SET_CONFIGURATION_PARAMETER;
			cmd[1] = parameter;
			cmd[2] = value;
			cmd[3] = 0x55;
			cmd[4] = 0x2A;
			UARTSend(uart, cmd, 5);

			cmd[0] =0;
			count = UartScanf(cmd, uart, UARTtimeOut);
			if (count ==1)
			{
				res = cmd[0];
				sprintf(response, "Set parameter Ok");
			}
		}
		xSemaphoreGive( xMutex );
	}
	return res;
}

/*****************************************************************************
** Function name:		motorForward
**
** Descriptions:		Set the specified motor to an indicated speed forward
**
** parameters:			integer motor id
** 						integer speed
** Returned value:		None
**
*****************************************************************************/
uint32_t motorForward(uint32_t motor, uint32_t speed)
{
	uint32_t ret = FALSE;
	if (uart < MAX_UART_NUMBER && xMutex != NULL)
	{
		xSemaphoreTake( xMutex, portMAX_DELAY );
		{
			if (speed > 255){
				speed = 255;
			}

			if (speed > 127)
			{
				// 8-bit mode: actual speed is (speed + 128)
				cmd[0] = motor == 0 ? QIK_MOTOR_M0_FORWARD_8_BIT : QIK_MOTOR_M1_FORWARD_8_BIT;
				cmd[1] = speed - 128;
			}
			else
			{
				cmd[0] = motor == 0 ? QIK_MOTOR_M0_FORWARD : QIK_MOTOR_M1_FORWARD;
				cmd[1] = speed;
			}
			UARTSend(uart, cmd, 2);
			ret = TRUE;
		}
		xSemaphoreGive( xMutex );
	}
	return ret;
}

/*****************************************************************************
** Function name:		motorReverse
**
** Descriptions:		Set the specified motor to an indicated speed backward
**
** parameters:			integer motor id
** 						integer speed
** Returned value:		None
**
*****************************************************************************/
uint32_t motorReverse(uint32_t motor, uint32_t speed)
{
	uint32_t ret = FALSE;
	if (uart < MAX_UART_NUMBER && xMutex != NULL)
	{
		xSemaphoreTake( xMutex, portMAX_DELAY );
		{
			if (speed > 255){
				speed = 255;
			}

			if (speed > 127)
			{
				// 8-bit mode: actual speed is (speed + 128)
				cmd[0] = motor == 0 ? QIK_MOTOR_M0_REVERSE_8_BIT : QIK_MOTOR_M1_REVERSE_8_BIT;
				cmd[1] = speed - 128;
			}
			else
			{
				cmd[0] = motor == 0 ? QIK_MOTOR_M0_REVERSE : QIK_MOTOR_M1_REVERSE;
				cmd[1] = speed;
			}
			UARTSend(uart, cmd, 2);
			ret = TRUE;
		}
		xSemaphoreGive( xMutex );
	}
	return ret;
}

/*****************************************************************************
** Function name:		motorCoast
**
** Descriptions:		Set the specified motor to coast by setting the motor
** 						outputs to high impedance, which lets the motor turn freely.

** parameters:			integer motor id
** Returned value:		None
**
*****************************************************************************/
uint32_t motorCoast(uint32_t motor)
{
	uint32_t ret = FALSE;
	if (uart < MAX_UART_NUMBER && xMutex != NULL)
	{
		xSemaphoreTake( xMutex, portMAX_DELAY );
		{
			cmd[0] = motor == 0 ? QIK_2S9V1_MOTOR_M0_COAST : QIK_2S9V1_MOTOR_M1_COAST;
			UARTSend(uart, cmd, 1);
			ret = TRUE;
		}
		xSemaphoreGive( xMutex );
	}
	return ret;
}
