/*
 * uartTools.c
 *
 *  Created on: 15/12/2012
 *      Author: Francec Pujol Contreras
 */

#include "FreeRTOS.h"
#include "task.h"
#include "semphr.h"
#include "type.h"

#include <stdio.h>
#include <string.h>
#include <stdarg.h>
#include <stdlib.h>

#include "uart.h"
#include "uartTools.h"

extern volatile uint32_t UART0Count;
extern volatile uint8_t UART0Buffer[BUFSIZE];

extern volatile uint32_t UART1Count;
extern volatile uint8_t UART1Buffer[BUFSIZE];

extern volatile uint32_t UART3Count;
extern volatile uint8_t UART3Buffer[BUFSIZE];

#define MAX_UART_NUMBER						( 4 )
#define RESPONSE_TIME_OUT 					( 10000 )
#define PRINTF_BUFF_LEN 					( 256 )

#define RESPONSE_BUFF_LEN 					( 512 )
static char string[RESPONSE_BUFF_LEN];
static int32_t defaultUart = -1;
static xSemaphoreHandle xMutex;


/*****************************************************************************
** Function name:		printfUartInit
**
** Descriptions:		Defines the default used UART and its baudrate.
** 						Also creates a semaphore for concurrent usage.
**
** parameters:			default uart, baudrate setup
** Returned value:		TRUE is successful
**
*****************************************************************************/
uint32_t printfUartInit(uint32_t useUart, uint32_t baudrate)
{
	uint32_t ret = FALSE;
	if (useUart <MAX_UART_NUMBER)
	{
		defaultUart = useUart;
		ret = UARTInit(defaultUart, baudrate); // init uart

		xMutex = xSemaphoreCreateMutex();

		// Verify that the semaphore was created successfully.
		if( ret == FALSE || xMutex == NULL )
			ret = FALSE;
	}
	return ret;
}

/*****************************************************************************
** Function name:		dUartPrintf
**
** Descriptions:		Writes the C string pointed by format to the default UART.
** 						If format includes format specifiers (subsequences
** 						beginning with %), the additional arguments following
** 						format are formatted and inserted in the resulting
** 						string replacing their respective specifiers.
** 						A semaphore allows concurrent usage.
**
** parameters:			char* string that contains the text to be written to
** 						default UART.
** Returned value:		None
**
*****************************************************************************/
void dUartPrintf(char* format,  ...)
{
	if(defaultUart != -1 && xMutex != NULL)
	{
		xSemaphoreTake( xMutex, portMAX_DELAY );
		{
			static char buffer[PRINTF_BUFF_LEN];
			va_list args;
			va_start (args, format);
			vsprintf (buffer,format, args);
			va_end (args);
			uint32_t len =  strlen(buffer);
			UARTSend(defaultUart, (uint8_t *)buffer ,len );
		}
		xSemaphoreGive( xMutex );
	}
}

/*****************************************************************************
** Function name:		dUartPrintfLn
**
** Descriptions:		Writes the C string pointed by format plus carry return
** 						and line feed chars to the default UART.
** 						If format includes format specifiers (subsequences
** 						beginning with %), the additional arguments following
** 						format are formatted and inserted in the resulting
** 						string replacing their respective specifiers.
** 						A semaphore allows concurrent usage.
**
** parameters:			char* string that contains the text to be written to
** 						default UART.
** Returned value:		None
**
*****************************************************************************/
void dUartPrintfLn(char* format,  ...)
{
	if(defaultUart != -1 && xMutex != NULL)
	{
		xSemaphoreTake( xMutex, portMAX_DELAY );
		{
			static char buffer[PRINTF_BUFF_LEN];
			va_list args;
			va_start (args, format);
			vsprintf (buffer,format, args);
			va_end (args);
			uint32_t len =  strlen(buffer);
			UARTSend(defaultUart, (uint8_t *)buffer ,len );
			UARTSend(defaultUart, (uint8_t *)"\r\n" ,2 );
		}
		xSemaphoreGive( xMutex );
	}
}

/*****************************************************************************
** Function name:		dUartPrintf
**
** Descriptions:		Writes the C string pointed by format to the UART.
** 						If format includes format specifiers (subsequences
** 						beginning with %), the additional arguments following
** 						format are formatted and inserted in the resulting
** 						string replacing their respective specifiers.
** 						A semaphore allows concurrent usage.
**
** parameters:			Integer, the UART id.
** 						char* string that contains the text to be written to
** 						specified UART.
** Returned value:		None
**
*****************************************************************************/
void uartPrintf(uint32_t uart, char* format,  ...)
{
	if(xMutex == NULL)
	{
		xMutex = xSemaphoreCreateMutex();
	}

	if(xMutex != NULL)
	{
		xSemaphoreTake( xMutex, portMAX_DELAY );
		{
			static char buffer[PRINTF_BUFF_LEN];
			va_list args;
			va_start (args, format);
			vsprintf (buffer,format, args);
			va_end (args);
			uint32_t len =  strlen(buffer);
			UARTSend(uart, (uint8_t *)buffer ,len );
		}
		xSemaphoreGive( xMutex );
	}
}

/*****************************************************************************
** Function name:		uartPrintfln
**
** Descriptions:		Send to the UART a string plus carry return
** 						and line feed char. The format string contains the text
** 						to be written that can optionally contain format
** 						specifiers (subsequences beginning with %) that are
** 						replaced by the values specified in subsequent additional
** 						arguments and formatted as requested.
**
** parameters:			Integer, the UART id.
** 						char* string that contains the text to be written to
** 						specified UART.
** Returned value:		None
**
*****************************************************************************/
void uartPrintfLn(uint32_t uart, char* format,  ...)
{
	if(xMutex == NULL)
	{
		xMutex = xSemaphoreCreateMutex();
	}

	if(xMutex != NULL)
	{
		xSemaphoreTake( xMutex, portMAX_DELAY );
		{
			static char buffer[PRINTF_BUFF_LEN];
			va_list args;
			va_start (args, format);
			vsprintf (buffer,format, args);
			va_end (args);
			uint32_t len =  strlen(buffer);
			UARTSend(uart, (uint8_t *)buffer ,len );
			UARTSend(uart, (uint8_t *)"\r\n" ,2 );
		}
		xSemaphoreGive( xMutex );
	}
}

/*****************************************************************************
** Function name:
**
** Descriptions:		Send ... value.
**
** parameters:			char* for the return message of the module response
** Returned value:		TRUE is successful
**
*****************************************************************************/
void uartClearBuff(uint32_t uartId)
{
	const portTickType xDelay5ms = 5 / portTICK_RATE_MS;
	static uint32_t oldCount;
	static uint32_t newCount;

	if (uartId ==0)
	{
		do{
			oldCount = newCount;
			vTaskDelay(xDelay5ms);
			newCount = UART0Count;
		}while(oldCount != newCount); 	// wait a little time while we have new chars

		LPC_UART0->IER = IER_THRE | IER_RLS;				// Disable RBR
		UART0Count = 0;
		LPC_UART0->IER = IER_THRE | IER_RLS | IER_RBR;		// Re-enable RBR
	}
	else if (uartId ==1)
	{
		do{
			oldCount = newCount;
			vTaskDelay(xDelay5ms);
			newCount = UART1Count;
		}while(oldCount != newCount); 	// wait a little time while we have new chars

		LPC_UART1->IER = IER_THRE | IER_RLS;				// Disable RBR
		UART1Count = 0;
		LPC_UART1->IER = IER_THRE | IER_RLS | IER_RBR;		// Re-enable RBR
	}
	else if (uartId ==3)
	{
		do{
			oldCount = newCount;
			vTaskDelay(xDelay5ms);
			newCount = UART3Count;
		}while(oldCount != newCount); 	// wait a little time while we have new chars

		LPC_UART3->IER = IER_THRE | IER_RLS;				// Disable RBR
		UART3Count = 0;
		LPC_UART3->IER = IER_THRE | IER_RLS | IER_RBR;		// Re-enable RBR
	}
}

/*****************************************************************************
** Function name:
**
** Descriptions:		Send ... value.
**
** parameters:			char* for the return message of the module response
** Returned value:		TRUE is successful
**
*****************************************************************************/
uint32_t uartResponse(uint32_t uart, char* key, uint32_t timeout, char* response, uint32_t responseSize)
{
	uint32_t ret = FALSE;
	const portTickType delayMiliseconds =  5 / portTICK_RATE_MS;
	uint32_t reading = timeout / delayMiliseconds;

	//vTaskDelay(250);	// wait sometime for receive some commands
	response[0] = '\0';
	while (reading>0)
	{
		reading--;

		if (uart ==0)
		{
			if( UART0Count != 0 )
			{
				LPC_UART0->IER = IER_THRE | IER_RLS;				// Disable RBR
				if(strlen(response)+ strlen((char*)UART0Buffer) <responseSize)
				{
					strncat(response, (char*)UART0Buffer, UART0Count);
				}
				else
				{
					strcpy (response, "Overflow !!\r\n");
					reading = 10;
					ret = FALSE;
				}
				UART0Count = 0;
				LPC_UART0->IER = IER_THRE | IER_RLS | IER_RBR;		// Re-enable RBR
				if(key != NULL)
				{
					if(strstr(response, key) != NULL)
					{
						reading = 10;
						ret = TRUE;
					}
				}
			}
		}
		else  if (uart ==1)
		{
			if( UART1Count != 0 )
			{
				LPC_UART1->IER = IER_THRE | IER_RLS;				// Disable RBR
				if(strlen(response)+ strlen((char*)UART1Buffer) <responseSize)
				{
					strncat(response, (char*)UART1Buffer, UART1Count);
				}
				else
				{
					strcpy (response, "Overflow !!\r\n");
					ret = FALSE;
				}
				UART1Count = 0;
				LPC_UART1->IER = IER_THRE | IER_RLS | IER_RBR;		// Re-enable RBR
				if(key != NULL)
				{
					if(strstr(response, key) != NULL)
					{
						reading = 10;
						ret = TRUE;
					}
				}
			}
		}
		else if (uart ==3)
		{
			if( UART3Count != 0 )
			{
				LPC_UART3->IER = IER_THRE | IER_RLS;				// Disable RBR
				if(strlen(response)+ strlen((char*)UART3Buffer) <responseSize)
				{
					strncat(response, (char*)UART3Buffer, UART3Count);
				}
				else
				{
					strcpy (response, "Overflow !!\r\n");
					ret = FALSE;
				}
				UART3Count = 0;
				LPC_UART3->IER = IER_THRE | IER_RLS | IER_RBR;		// Re-enable RBR
				if(key != NULL)
				{
					if(strstr(response, key) != NULL)
					{
						reading = 10;
						ret = TRUE;
					}
				}
			}
		}
		vTaskDelay(delayMiliseconds);
	}

	uartClearBuff(uart);
	return ret;
}

/*****************************************************************************
** Function name:
**
** Descriptions:		Send ... value.
**
** parameters:			char* for the return message of the module response
** Returned value:		TRUE is successful
**
*****************************************************************************/
uint32_t checkResponse(uint32_t uart, char* value, char* message, char* response)
{
	uint32_t ret;
	if(uartResponse(uart, value, RESPONSE_TIME_OUT, string, RESPONSE_BUFF_LEN) == TRUE){
		sprintf(response, "%s%s", response, message);
		ret = TRUE;
	}
	else{
		sprintf(response, "Error: '%s'", string);
		ret = FALSE;
	}
	return ret;
}


/*****************************************************************************
** Function name:
**
** Descriptions:		Send ... value.
**
** parameters:			uint8_t* for the return message of the module response
** Returned value:		TRUE is successful
**
*****************************************************************************/
uint32_t UartScanf(uint8_t* string, uint32_t uart, uint32_t timeout)
{
	static int oldCount;
	static int newCount;
	const portTickType xDelay5ms = 5 / portTICK_RATE_MS;

	if(uart < MAX_UART_NUMBER && xMutex != NULL)
	{
		xSemaphoreTake( xMutex, portMAX_DELAY );
		{
			if (uart ==0)
			{
				UART0Count = 0;									// clean buff
				LPC_UART0->IER = IER_THRE | IER_RLS | IER_RBR; 	// enable interrupt RX UART0
				vTaskDelay(timeout);							// wait sometime for receive some commands
				do{
					oldCount = newCount;
					vTaskDelay(xDelay5ms);
					newCount = UART0Count;
				}while(oldCount != newCount); 					// wait a little time while we have new chars
				LPC_UART0->IER = IER_THRE | IER_RLS;			// disable interrupt RX UART0

				strncpy((char*)string, (char*)UART0Buffer, newCount );
			}
			else if(uart ==1)
			{
				UART1Count = 0;									// clean buff
				LPC_UART1->IER = IER_THRE | IER_RLS | IER_RBR; 	// enable interrupt RX UART1
				vTaskDelay(timeout);							// wait sometime for receive some commands
				do{
					oldCount = newCount;
					vTaskDelay(xDelay5ms);
					newCount = UART1Count;
				}while(oldCount != newCount); 					// wait a little time while we have new chars
				LPC_UART1->IER = IER_THRE | IER_RLS;			// disable interrupt RX UART1

				strncpy((char*)string, (char*)UART1Buffer, newCount );
			}
			else if(uart ==3)
			{
				UART3Count = 0;									// clean buff
				LPC_UART3->IER = IER_THRE | IER_RLS | IER_RBR; 	// enable interrupt RX UART3
				vTaskDelay(timeout);							// wait sometime for receive some commands
				do{
					oldCount = newCount;
					vTaskDelay(xDelay5ms);
					newCount = UART3Count;
				}while(oldCount != newCount); 					// wait a little time while we have new chars
				LPC_UART3->IER = IER_THRE | IER_RLS;			// disable interrupt RX UART3

				strncpy((char*)string, (char*)UART3Buffer, newCount );
			}

			string[newCount] = 0;							// add end string. Now we have a string
			// uartClearBuff(uart);  //!!! CESC Atencio !!
		}
		xSemaphoreGive( xMutex );
	}
	return newCount;
}

/*****************************************************************************
** Function name:
**
** Descriptions:		Send ... value.
**
** parameters:			uint8_t* for the return message of the module response
** Returned value:		TRUE is successful
**
*****************************************************************************/
uint32_t dUartScanf(uint8_t* string, uint32_t timeout)
{
	return UartScanf(string, defaultUart, timeout);
}
