/*
 * main.c
 *
 *  Created on: 15/12/2012
 *      Author: Francec Pujol Contreras
 */

/* FreeRTOS.org includes. */
#include "FreeRTOS.h"
#include "task.h"
#include "queue.h"
#include "type.h"

#include <stdio.h>
#include <string.h>
#include <stdarg.h>

#include "uart.h"
#include "uartTools.h"
#include "wifly.h"
#include "arplab.h"
#include "qik2s9v1.h"
#include "pwm.h"
#include "adc.h"

#define RESPONSE_BUFF_LEN (256)
static char response[RESPONSE_BUFF_LEN];

// PWM
#define CHANNEL_NUM			(1)
extern volatile uint32_t match_counter1;

// ADC
extern volatile uint32_t ADC0Value[ADC_NUM];
extern volatile uint32_t ADC0IntDone;
extern void TargetResetInit(void);
#define ADC_COUNT_MAX 4095
#define SUPPLY_VOLTAGE 3.30

// Input
#define PIN_JUMPER_AUTOMATIC (1 << 7)            // input P2.7

// The task functions.
void vTaskRemoteControl( void *pvParameters );
void vTaskAutomaticControl( void *pvParameters );
void vTaskTelnetConsumer( void *pvParameters );
void vTaskPWM( void *pvParameters );
void vTaskADC( void *pvParameters );
void initWiFly();
void initQik(uint32_t runTest);

static uint32_t LedPower = 	0;
static uint32_t ultrasonicValue;
static double ultrasonicCm;

static uint32_t wiFlyBaudrate = BAUD_57600;
static uint32_t termBaudrate = 	BAUD_115200;
static uint32_t qikBaudrate = 	BAUD_9600;
static double obstaclesStopCm = 	25.0f;
static uint32_t SendStatusQueue = FALSE;
const uint32_t testMotors = FALSE;

#define UART_DEBUG 			(UART1)
#define UART_WIFLY 			(UART0)
#define UART_QIK			(UART3)

#define WLAN_SSID 			"WLAN_SSID"
#define WLAN_KEY 			"WLAN_KEY"

// Declare a variable of type xQueueHandle. This is used to store the queue
//that is accessed by all WiFly tasks.
xQueueHandle xQueueReceiveWifly;
xQueueHandle xQueueSendWifly;


/*****************************************************************************
** Function name:		Main
**
** Descriptions:		Entry point.Read GPIO2 input jumper and star tasks
** 						for Automatic or Remote control. Also run tasks
** 						for PWM and ADC ports.
**
** parameters:			None
** Returned value:		None
**
*****************************************************************************/
int main( void )
{
    xQueueReceiveWifly = xQueueCreate( 10, sizeof( xMessage) );
    xQueueSendWifly = xQueueCreate( 10, sizeof( xMessage) );

	if(	xQueueReceiveWifly != NULL &&
		xQueueSendWifly != NULL )
	{
		LPC_GPIO2->FIODIR &=~PIN_JUMPER_AUTOMATIC;        //set input
		LPC_GPIO2->FIOSET |= PIN_JUMPER_AUTOMATIC;        //set high
		//LPC_GPIO2->FIOCLR |= PIN_JUMPER_AUTOMATIC;		//set low
		if(LPC_GPIO2->FIOPIN & PIN_JUMPER_AUTOMATIC)        //get pin
		{
			//pin is still high
			LedPower =0;
			xTaskCreate( vTaskRemoteControl, "Task 1", 240, NULL, 1, NULL );
			xTaskCreate( vTaskTelnetConsumer, "Task 2", 240, NULL, 1, NULL );

		}
		else
		{
			//pin it's pulled low
			LedPower = PWM_CYCLE *75;
			xTaskCreate( vTaskAutomaticControl, "Task 1", 240, NULL, 1, NULL );
		}

		xTaskCreate( vTaskPWM, "Task 3", 240, NULL, 1, NULL );
		xTaskCreate( vTaskADC, "Task 4", 240, NULL, 1, NULL );
		// Start the scheduler so our tasks start executing.
		vTaskStartScheduler();
	}
	else
	{
		/* The queue could not be created. */
	}

	/* If all is well we will never reach here as the scheduler will now be
	running.  If we do reach here then it is likely that there was insufficient
	heap available for the idle task to be created. */
	for( ;; );
	return 0;
}

/*****************************************************************************
** Function name:		vTaskRemoteControl
**
** Descriptions:		Task for remote control
**
** parameters:			void * Not used
** Returned value:		None
**
*****************************************************************************/
void vTaskRemoteControl( void *pvParameters )
{
	printfUartInit(UART_DEBUG, termBaudrate);
	dUartPrintfLn("Welcome Remote Control");

	initQik(testMotors);

	initWiFly();
	wiFlyTelnetServer(xQueueReceiveWifly, xQueueSendWifly , response);
	dUartPrintfLn(response);

	dUartPrintfLn("Telnet end.");

	wiFlyOpen(response);
	dUartPrintfLn(response);
	wiFlyClose(response);
	dUartPrintfLn(response);

	dUartPrintfLn("WiFly commands over Debug UART");
	while(TRUE)
	{
		const uint32_t TimeOut =1;
		static char telnetMsg[512];
		UartScanf(telnetMsg, UART_DEBUG, TimeOut);
		if( strlen((char*)telnetMsg)>0){
			uartPrintf(UART_WIFLY, telnetMsg);
		}
		UartScanf(telnetMsg, UART_WIFLY, TimeOut);
		if( strlen((char*)telnetMsg)>0){
			uartPrintf(UART_DEBUG, telnetMsg);
		}
	}
}


/*****************************************************************************
** Function name:		vTaskAutomaticControl
**
** Descriptions:		Task for automatic control. Moves the Vehicle forward
** 						or backward searching the best signal strength.
**
** parameters:			void * Not used
** Returned value:		None
**
*****************************************************************************/
void vTaskAutomaticControl( void *pvParameters )
{
	printfUartInit(UART_DEBUG, termBaudrate);
	dUartPrintfLn("Welcome Automatic Control");

	initQik(testMotors);
	initWiFly();

	motorForward(MOTOR_M1, 0);

	static double lastUltrasonicCm;

	lastUltrasonicCm = ultrasonicCm;
	int32_t lastRssi;
	int32_t rssi;

	while(TRUE)
	{
		lastRssi = rssi;
		rssi = wiFlyRssi(response);
		if((PWM_CYCLE  *75/100) -((-rssi-40)*47) < 1){
			LedPower =1;
		}
		else{
			LedPower = (PWM_CYCLE  *75/100) -((-rssi-40)*47);
		}

		dUartPrintfLn("Led power: %i (%i db)", LedPower, rssi);
		if(lastUltrasonicCm < obstaclesStopCm && ultrasonicCm < obstaclesStopCm)
		{
			dUartPrintfLn("Stop, obstacle at %.2f cm.", ultrasonicCm);
			motorForward(MOTOR_M0, 0);
			motorCoast(MOTOR_M0);
		}
		else
		{
			if(rssi != lastRssi)
			{
				if(rssi>lastRssi)
				{
					motorForward(MOTOR_M0, 30);
				}
				else
				{
					motorReverse(MOTOR_M0, 30);
				}
			}
			else
			{
				motorForward(MOTOR_M0, 0);
				motorCoast(MOTOR_M0);
			}
		}

		lastUltrasonicCm = ultrasonicCm;
		vTaskDelay(2000 / portTICK_RATE_MS);
	}
}

/*****************************************************************************
** Function name:		vTaskTelnetConsumer
**
** Descriptions:		Task for remote control messages. Receive and execute
** 						remote commands and send the responses to the telnet
** 						client.
**
** parameters:			void * Not used
** Returned value:		None
**
*****************************************************************************/
void vTaskTelnetConsumer( void *pvParameters )
{
	portBASE_TYPE xStatus;
	uint32_t ret;
	uint32_t speed;
	static char responseToWifly[32];
	static char motorCommand[30];
	static xMessage xStructToReceive = { 0, "" };
	static xMessage xStructToSend = { 0, "" };
	static double lastUltrasonicCm;

	const uint32_t speedLimit = 90;
	const uint32_t direccionLimit = 40;

	const portTickType xTicksToWait = 100 / portTICK_RATE_MS;
	uint32_t messagesWatchDog =0;
	uint32_t SonarControl = TRUE;

	// Wait until Producer init WiFly Wlan
	const portTickType xDelay = 12000 / portTICK_RATE_MS;
	vTaskDelay(xDelay);

	printfUartInit(UART_DEBUG, termBaudrate);
	// This task is also defined within an infinite loop.
	for( ;; )
	{

		// As this task unblocks immediately that data is written to the queue this
		// call should always find the queue empty.
		if( uxQueueMessagesWaiting( xQueueReceiveWifly ) != 0 )
		{
			if (SendStatusQueue){
				dUartPrintfLn("Queue have %i items", uxQueueMessagesWaiting( xQueueReceiveWifly ));
			}
		}

		xStatus = xQueueReceive( xQueueReceiveWifly, &xStructToReceive, xTicksToWait );
		if( xStatus == pdPASS )
		{
			// Data was successfully received from the queue, print out the received value.
			//dUartPrintfLn("Queue received:  %s (size %i)", xStructToReceive.message, (uint32_t)xStructToReceive.messageLength);
			responseToWifly[0] ='\0';
			sscanf(xStructToReceive.message, "$$%s", motorCommand);
			if(strlen(motorCommand)>0)
			{
				sprintf(responseToWifly, "Unknown command.");
				messagesWatchDog =0;
				if(strstr(motorCommand, "f:") != NULL)
				{
					char value[10];
					sscanf(motorCommand, "f:%99[^\r\n]", value);
					speed = atoi(value) % speedLimit;
					motorForward(MOTOR_M0, speed);
					sprintf(responseToWifly, "Ok. Forward Speed:%i", speed);
				}
				else if(strstr(motorCommand, "b:") != NULL)
				{
					char value[10];
					sscanf(motorCommand, "b:%99[^\r\n]", value);
					speed = atoi(value) % speedLimit;
					motorReverse(MOTOR_M0, speed);
					sprintf(responseToWifly, "Ok. Backward Speed:%i", speed);
				}
				else if(strstr(motorCommand, "fd:") != NULL)
				{
					char value[10];
					sscanf(motorCommand, "fd:%99[^\r\n]", value);
					speed = atoi(value) % speedLimit;
					motorForward(MOTOR_M1, speed);
					sprintf(responseToWifly, "Ok.Forward direction motor:%i", speed);
				}
				else if(strstr(motorCommand, "bd:") != NULL)
				{
					char value[10];
					sscanf(motorCommand, "bd:%99[^\r\n]", value);
					speed = atoi(value) % speedLimit;
					motorReverse(MOTOR_M1, speed);
					sprintf(responseToWifly, "Ok.Backward direction motor:%i", speed);
				}
				else if(strstr(motorCommand, "l:") != NULL)
				{
					char value[10];
					sscanf(motorCommand, "l:%99[^\r\n]", value);
					speed = atoi(value) % direccionLimit;
					motorReverse(MOTOR_M1, speed);
					sprintf(responseToWifly, "Ok. Left power:%i", speed);
				}
				else if(strstr(motorCommand, "r:") != NULL)
				{
					char value[10];
					sscanf(motorCommand, "r:%99[^\r\n]", value);
					speed = atoi(value) % direccionLimit;
					motorForward(MOTOR_M1, speed);
					sprintf(responseToWifly, "Ok. Right power:%i", speed);
				}
				else if(strstr(motorCommand, "stop") != NULL)
				{
					motorForward(MOTOR_M0, 0);
					motorForward(MOTOR_M1, 0);
					sprintf(responseToWifly, "Ok. Stop");
				}
				else if(strstr(motorCommand, "sonar") != NULL)
				{
					sprintf(responseToWifly, "value:%i", ultrasonicValue);
					sprintf(responseToWifly, "cm:%.2f", ultrasonicCm); //ultrasonicValue);
				}
				else if(strstr(motorCommand, "obstacles\r\n") != NULL)
				{
					SonarControl = !SonarControl;
					if(SonarControl == TRUE){
						sprintf(responseToWifly, "Ok. Stop on obstacle detection");
					}
					else{
						sprintf(responseToWifly, "Ok. Ignore obstacles");
					}
				}
				else if(strstr(motorCommand, "obstacles:") != NULL)
				{
					char value[10];
					sscanf(motorCommand, "obstacles:%99[^\r\n]", value);
					obstaclesStopCm = atoi(value);
					SonarControl = TRUE;
					sprintf(responseToWifly, "Ok. Ok. Stop on obstacle at %i cm", obstaclesStopCm);
				}
				else if(strstr(motorCommand, "status") != NULL)
				{
					SendStatusQueue = !SendStatusQueue;
					if(SendStatusQueue == TRUE){
						sprintf(responseToWifly, "Ok. Send status messages");
					}
					else{
						sprintf(responseToWifly, "Ok. Don't send status messages");
					}
				}
				else if(strstr(motorCommand, "led:") != NULL)
				{
					char value[10];
					sscanf(motorCommand, "led:%99[^\r\n]", value);
					LedPower = atoi(value);
					sprintf(responseToWifly, "Ok. Led power:%i", LedPower);
				}
				else if(strstr(motorCommand, "init") != NULL)
				{
					LedPower =0;
					SendStatusQueue = FALSE;
					SonarControl =TRUE;
					motorForward(MOTOR_M0,0);
					motorForward(MOTOR_M1,0);
					motorCoast(MOTOR_M0);
					motorCoast(MOTOR_M1);
					sprintf(responseToWifly, "Ok. Init default values", LedPower);
				}

				dUartPrintfLn(responseToWifly);
				xStructToSend.messageLength = strlen(responseToWifly);
				strcpy(xStructToSend.message, responseToWifly);
				do{
					xStatus = xQueueSendToBack( xQueueSendWifly, ( void * ) &xStructToSend, 0);
				} while  (xStatus != pdPASS );
			}
		}
		else
		{
			if (SonarControl &&
				ultrasonicCm < obstaclesStopCm &&
				lastUltrasonicCm < obstaclesStopCm)
			{
				motorForward(MOTOR_M0,0);
				motorForward(MOTOR_M1,0);
				motorCoast(MOTOR_M0);
				motorCoast(MOTOR_M1);
				dUartPrintfLn("Stop, obstacle at %.2f cm.", ultrasonicCm);
				if(SendStatusQueue)
				{
					sprintf(responseToWifly, "Stop, obstacle at %.2f cm.", ultrasonicCm);
					xStructToSend.messageLength = strlen(responseToWifly);
					strcpy(xStructToSend.message, responseToWifly);
					do{
						xStatus = xQueueSendToBack( xQueueSendWifly, ( void * ) &xStructToSend, 0);
					} while  (xStatus != pdPASS );
				}
			}

			lastUltrasonicCm = ultrasonicCm;
			messagesWatchDog++;
			if( messagesWatchDog>40 ){
				messagesWatchDog =0;
				// We did not receive anything from the queue even after waiting for 4 s.
				dUartPrintfLn("Commands queue empty.");
				motorForward(MOTOR_M0,0);
				motorForward(MOTOR_M1,0);
				motorCoast(MOTOR_M0);
				motorCoast(MOTOR_M1);

				if(SendStatusQueue)
				{
					sprintf(responseToWifly, "Commands queue empty.");
					xStructToSend.messageLength = strlen(responseToWifly);
					strcpy(xStructToSend.message, responseToWifly);
					do{
						xStatus = xQueueSendToBack( xQueueSendWifly, ( void * ) &xStructToSend, 0);
					} while  (xStatus != pdPASS );
				}
			}
		}
	}
}

/*****************************************************************************
** Function name:		vTaskADC
**
** Descriptions:		Task for reading sonar data and calculate obstacles
** 						distance.
**
** parameters:			void * Not used
** Returned value:		Updates the globals ultrasonicValue and ultrasonicCm
**
*****************************************************************************/
void vTaskADC( void *pvParameters )
{
	const portTickType readDelay = 125 / portTICK_RATE_MS;
	const double cmPerInch =  0.3937f;

	ADCInit( ADC_CLK ); // Initialize ADC
	const portTickType xDelay = 8000 / portTICK_RATE_MS;
	vTaskDelay(xDelay);

	while(TRUE)
	{
		vTaskDelay(readDelay);
		uint32_t i;
		for ( i = 0; i < ADC_NUM; i++ )
		{
		  ADC0Read( i );
		  while ( !ADC0IntDone );
		  ADC0IntDone = 0;
		}
		ultrasonicValue = ADC0Value[0];
		//The formula for the voltage scaling on an LV‑MaxSonar‑EZ1 is:
		//[(Vcc/512) = Vi]
		//Vcc = Supplied Voltage
		//Vi = Volts per inch (Scaling)
		//cm = in / 0,3937
		ultrasonicCm = (((double)ultrasonicValue /ADC_COUNT_MAX) * 512) / cmPerInch;

		//double ultrasonicVolts = ultrasonicValue *SUPPLY_VOLTAGE/ADC_COUNT_MAX;
		//double voltsPerInch = SUPPLY_VOLTAGE / 512;
		//ultrasonicCm = (ultrasonicVolts / voltsPerInch) / cmPerInch;
	}
}


/*****************************************************************************
** Function name:		vTaskPWM
**
** Descriptions:		Task for update status led with PWM output.
**
** parameters:			void * Not used
** Returned value:		Updates the global LedPower
**
*****************************************************************************/
void vTaskPWM( void *pvParameters )
{
	uint32_t cycle = PWM_CYCLE, offset = PWM_CYCLE * 0/6;
	volatile unsigned long ul;
	volatile long delta =1;
	//SystemClockUpdate(); 	// SystemClockUpdate() updates the SystemFrequency variable

	if ( PWM_Init( CHANNEL_NUM, 0 ) != TRUE )
	{
		dUartPrintfLn("PWM Fatal error.");
		while( TRUE );			// fatal error
	}

	PWM_Set( CHANNEL_NUM, cycle, offset );
	PWM_Start( CHANNEL_NUM );

	while ( TRUE )
	{
		vTaskDelay( 1 / portTICK_RATE_MS);

		if(LedPower ==0)
		{
			offset+=delta;
			if (offset > PWM_CYCLE *65/100) delta =-1;
			if (offset < PWM_CYCLE *1/100) delta =1;
		}
		else
		{
			offset = LedPower % PWM_CYCLE;
		}

		if ( match_counter1 != 0 )
		{
		  match_counter1 = 0;
		  PWM_Set( CHANNEL_NUM, cycle, offset );
		}
	}
}

/*****************************************************************************
** Function name:		initWiFly
**
** Descriptions:		Initializes the WiFi board
**
** parameters:			None
** Returned value:		None
**
*****************************************************************************/
void initWiFly()
{
	dUartPrintfLn("Init WiFly");
	wiFlyInit(UART_WIFLY, wiFlyBaudrate);

	wiFlyOpen(response);
	dUartPrintfLn(response);

	wiFlyLites(response);
	dUartPrintfLn(response);

	wiFlyVersion(response);
	dUartPrintfLn("Version: %s", response);

	wiFlyDisconnect(response);
	dUartPrintfLn(response);

	wiFlyConnect(WPA2PSK, WLAN_SSID,WLAN_KEY, response);
	dUartPrintfLn(response);

	wiFlyGetIp(response);
	dUartPrintfLn("IP: %s", response);

	wiFlyLites(response);
	dUartPrintfLn(response);
}

/*****************************************************************************
** Function name:		initQik
**
** Descriptions:		Initializes the motors control Qik 2s9v1 board
**
** parameters:			None
** Returned value:		Integer, TRUE for run a motors test
**
*****************************************************************************/
void initQik(uint32_t runTest)
{
	dUartPrintfLn("Init Qik2s9v1");
	qikInit(UART_QIK, qikBaudrate);
	QikVersion(response);
	dUartPrintfLn("Get Firmware Version. Return %s", response);

	// #define QIK_CONFIG_DEVICE_ID                        0
	// uint32_t ret = setConfigurationParameter(QIK_CONFIG_DEVICE_ID, 9, response);
	// dUartPrintfLn("Set DeviceID 9. Return %i. %s", ret , response);

	uint32_t devId = QikGetDeviceId(response);
	dUartPrintfLn("Get device id %i", devId);

	uint32_t pwm = QikGetPWMparameter(response);
	dUartPrintfLn("Get PWM Parameter %i", pwm);

	uint32_t shutdown = QikGetShutDownOnError(response);
	if (shutdown == 1)
		dUartPrintfLn("Get shutdown on Error True");
	else
		dUartPrintfLn("Get shutdown on Error False");

	uint32_t timeout = QikGetSerialTimeout(response);
	dUartPrintfLn("Get Serial Timeout %i", timeout);

	uint32_t err = QikGetError(response);
	dUartPrintfLn("Get error code: 0x%X", err);

	if(runTest == TRUE)
	{
		motorForward(MOTOR_M0, 12);
		vTaskDelay(1000 / portTICK_RATE_MS);

		motorForward(MOTOR_M0, 20);
		vTaskDelay(1000 / portTICK_RATE_MS);

		motorForward(MOTOR_M0, 30);
		vTaskDelay(1000 / portTICK_RATE_MS);

		motorForward(MOTOR_M0, 12);
		vTaskDelay(1000 / portTICK_RATE_MS);

		motorForward(MOTOR_M0,0);
		motorCoast(MOTOR_M0);

		motorForward(MOTOR_M1, 20);
		vTaskDelay(1000 / portTICK_RATE_MS);

		motorReverse(MOTOR_M1, 20);
		vTaskDelay(1000 / portTICK_RATE_MS);

		motorForward(MOTOR_M1,0);
	}
}

/*-----------------------------------------------------------*/

void vApplicationMallocFailedHook( void )
{
	/* This function will only be called if an API call to create a task, queue
	or semaphore fails because there is too little heap RAM remaining. */
	for( ;; );
}
/*-----------------------------------------------------------*/

void vApplicationStackOverflowHook( xTaskHandle *pxTask, signed char *pcTaskName )
{
	/* This function will only be called if a task overflows its stack.  Note
	that stack overflow checking does slow down the context switch
	implementation. */
	for( ;; );
}
/*-----------------------------------------------------------*/

void vApplicationIdleHook( void )
{
	/* This example does not use the idle hook to perform any processing. */
}
/*-----------------------------------------------------------*/

void vApplicationTickHook( void )
{
	/* This example does not use the tick hook to perform any processing. */
}


