/*
 * wifly.c
 *
 *  Created on: 03/11/2012
 *      Author: Francec Pujol Contreras
 */

/* FreeRTOS.org includes. */
#include "FreeRTOS.h"
#include "task.h"
#include "semphr.h"
#include "queue.h"
#include "type.h"

#include <stdio.h>
#include <stdarg.h>
#include <stdlib.h>
#include <string.h>

#include "uart.h"
#include "uartTools.h"
#include "wifly.h"

#define RST_WIFLY_PIN 						( 22 )
#define RESPONSE_TIME_OUT 					( 10000 )

#define RESPONSE_BUFF_LEN 					( 512 )
static char string[RESPONSE_BUFF_LEN];
static uint32_t uart = 99999;

/* Declare a variable of type xSemaphoreHandle.  This is used to reference the
mutex type semaphore that is used to ensure mutual exclusive access to WiFly. */
static xSemaphoreHandle xMutex;


/*****************************************************************************
** Function name:		wiFlyReset
**
** Descriptions:		Hardware reset the WiFly device.
** 						J6-24 (P0.22) of LPC1769 pin
**
** parameters:			None
** Returned value:		None
**
*****************************************************************************/
void wiFlyReset()
{
	// Initialize pin
	LPC_PINCON->PINSEL1 &= ( ~( 3 << 12 ) );
	LPC_GPIO0->FIODIR |= ( 1 << RST_WIFLY_PIN );


	// it made reset at Wifly
	LPC_GPIO0->FIOCLR = 1 << RST_WIFLY_PIN;
	const portTickType xDelay1ms = 1 / portTICK_RATE_MS;
	vTaskDelay(xDelay1ms); // min 160us
	LPC_GPIO0->FIOSET = 1 << RST_WIFLY_PIN;
}

/*****************************************************************************
** Function name:		wiFlyInit
**
** Descriptions:		Defines the used uart, its baudrate and do a reset
** 						in WiFly device. This operation produce a seconds 3
** 						delay.
**
** parameters:			uart, baudrate
** Returned value:		TRUE is successful
**
*****************************************************************************/
uint32_t wiFlyInit(uint32_t useUart, uint32_t baudrate)
{
	const portTickType xDelay3s = 3000 / portTICK_RATE_MS;	
	uint32_t ret = FALSE;

	if (useUart < MAX_UART_NUMBER)
	{
		uart = useUart;
		UARTInit(uart, baudrate);

		wiFlyReset();
		vTaskDelay(xDelay3s);
		ret = TRUE;

		xMutex = xSemaphoreCreateMutex();

		// Verify that the semaphore was created successfully.
		if( ret == FALSE || xMutex == NULL )
			ret = FALSE;
	}
	return ret;
}

/*****************************************************************************
** Function name:		wiFlyOpen
**
** Descriptions:		Entering Command Mode. To configure the parameters
** 						and/or to view the current configuration, the module
** 						must be put into command mode.
**
** parameters:			char* for the return message of the module response
** Returned value:		TRUE is successful
**
*****************************************************************************/
uint32_t wiFlyOpen(char* response)
{

	uint32_t ret = FALSE;
	if (uart < MAX_UART_NUMBER && xMutex != NULL)
	{
		xSemaphoreTake( xMutex, portMAX_DELAY );
		{
			const portTickType xDelay250ms = 250 / portTICK_RATE_MS;
			uartPrintf(uart, "$$$");
			vTaskDelay(xDelay250ms);
			response[0] = '\0';
			ret = checkResponse(uart, "CMD\r\n", "cmd Ok", response);
		}
		xSemaphoreGive( xMutex );
	}
	return ret;
}

/*****************************************************************************
** Function name:		wiFlyClose
**
** Descriptions:		disconnect a TCP connection.
**
** parameters:			char* for the return message of the module response
** Returned value:		TRUE is successful
**
*****************************************************************************/
uint32_t wiFlyClose(char* response)
{
	uint32_t ret = FALSE;
	if (uart < MAX_UART_NUMBER && xMutex != NULL)
	{
		xSemaphoreTake( xMutex, portMAX_DELAY );
		{
			uartPrintfLn(uart, "\r\nclose");
			response[0] = '\0';
			ret =checkResponse(uart, "*CLOS*", "close Ok", response);
		}
		xSemaphoreGive( xMutex );
	}
	return ret;
}

/*****************************************************************************
** Function name:		wiFlyExit
**
** Descriptions:		Exists Command Mode and entering Data Mode.
** 						In this mode, the module is ready to either accept
** 						incoming connections or initiate outgoing connections.
**
** parameters:			char* for the return message of the module response
** Returned value:		TRUE is successful
**
*****************************************************************************/
uint32_t wiFlyExit(char* response)
{
	uint32_t ret = FALSE;
	if (uart < MAX_UART_NUMBER && xMutex != NULL)
	{
		xSemaphoreTake( xMutex, portMAX_DELAY );
		{
			uartPrintfLn(uart, "\r\nexit");
			response[0] = '\0';
			ret =checkResponse(uart, "EXIT", "exit Ok", response);
		}
		xSemaphoreGive( xMutex );
	}
	return ret;
}

/*****************************************************************************
** Function name:		wiFlySave
**
** Descriptions:		save Saves the configuration
** 						to “config” (the default file).
**
** parameters:			char* for the return message of the module response
** Returned value:		TRUE is successful
**
*****************************************************************************/
uint32_t wiFlySave(char* response)
{
	uint32_t ret = FALSE;
	if (uart < MAX_UART_NUMBER && xMutex != NULL)
	{
		xSemaphoreTake( xMutex, portMAX_DELAY );
		{
			uartPrintfLn(uart, "save");
			response[0] = '\0';
			ret = checkResponse(uart, "Storing in config", "Save Ok", response);
		}
		xSemaphoreGive( xMutex );
	}
	return ret;
}

/*****************************************************************************
** Function name:		wiFlyReboot
**
** Descriptions:		forces a reboot of the device (similar to power cycle)
**						After the module reboots the settings take effect
**
** parameters:			char* for the return message of the module response
** Returned value:		TRUE is successful
**
*****************************************************************************/
uint32_t wiFlyReboot(char* response)
{
	uint32_t ret = FALSE;
	if (uart < MAX_UART_NUMBER && xMutex != NULL)
	{
		xSemaphoreTake( xMutex, portMAX_DELAY );
		{
			const portTickType xDelay2s = 2000 / portTICK_RATE_MS;
			uartPrintfLn(uart, "reboot");
			response[0] = '\0';
			vTaskDelay(xDelay2s);
			ret = checkResponse(uart, "*Reboot*", "Reboot Ok", response);
		}
		xSemaphoreGive( xMutex );
	}
	return ret;
}

/*****************************************************************************
** Function name:		wiFlyFactoryReset
**
** Descriptions:		Loads factory defaults into the WiFly RAM configuration. 
**						After this command the WiFly module is be rebooted 
**						for settings to take effect.
**
** parameters:			uart, debug message
** Returned value:		TRUE if response is Ok. 
**
*****************************************************************************/
uint32_t wiFlyFactoryReset(char* response)
{
	uint32_t ret = FALSE;
	if (uart < MAX_UART_NUMBER && xMutex != NULL)
	{
		xSemaphoreTake( xMutex, portMAX_DELAY );
		{
			const portTickType xDelay1s = 1000 / portTICK_RATE_MS;
			uartPrintfLn(uart, "factory RESET");
			response[0] = '\0';
			checkResponse(uart, ">", "factory RESET Ok\r\n", response);

			uartPrintfLn(uart, "set sys printlvl 0");
			checkResponse(uart, "AOK", "Quiet mode Ok\r\n", response);

			uartPrintfLn(uart, "save");
			checkResponse(uart, "Storing in config", "Save Ok\r\n", response);

			vTaskDelay(xDelay1s);
			uartPrintfLn(uart, "reboot");
			vTaskDelay(xDelay1s);
			UARTInit(uart, BAUD_9600);
			ret = checkResponse(uart, "*Reboot*", "Reboot Ok", response);
		}
		xSemaphoreGive( xMutex );
	}
	return ret;
}

/*****************************************************************************
** Function name:		wiFlyLites
**
** Descriptions:		lites Blinks LEDs on RN-134 Surf board and
** 						RN-370 WiFly serial adapter.
**						Issue the same command to stop blinking LEDs.
**
** parameters:			char* for the return message of the module response
** Returned value:		TRUE is successful
**
*****************************************************************************/
uint32_t wiFlyLites(char* response)
{
	uint32_t ret = FALSE;
	if (uart < MAX_UART_NUMBER && xMutex != NULL)
	{
		xSemaphoreTake( xMutex, portMAX_DELAY );
		{
			uartPrintfLn(uart, "lites");
			response[0] = '\0';
			ret = checkResponse(uart, "\r\n", "Lites Ok", response);
		}
		xSemaphoreGive( xMutex );
	}
	return ret;
}

/*****************************************************************************
** Function name:		wiFlyGetIp
**
** Descriptions:		Returns the current IP address value.
**
** parameters:			char* for the module current IP address
** Returned value:		TRUE is successful
**
*****************************************************************************/
uint32_t wiFlyGetIp(char *response)
{
	uint32_t ret = FALSE;
	if (uart < MAX_UART_NUMBER && xMutex != NULL)
	{
		xSemaphoreTake( xMutex, portMAX_DELAY );
		{
			uartPrintfLn(uart, "get ip");
			response[0] = '\0';
			if(uartResponse(uart, "IP=", RESPONSE_TIME_OUT, string, RESPONSE_BUFF_LEN) == TRUE)
			{
				sprintf(response, "Get Ip Ok\r\n");
				char *search = "\r\n";
				char *token;
				token = strtok((char*)string, search);

				while(token != NULL)
				{
					if(strstr(token, "IP=") != NULL)
					{
						char value[20];
						sscanf(token, "IP=%99[^:]", value);
						sprintf(response, value);
						ret = TRUE;
					}
					token = strtok(NULL, search);
				}
			}
		}
		xSemaphoreGive( xMutex );
	}
	return ret;
}

/*****************************************************************************
** Function name:		wiFlyPing
**
** Descriptions:		Ping and stop after sending count ECHO_REQUEST packets.
**
** parameters:			char* of the remote host name or IP address
** 						Number of packets to send
** 						char* for the return message of the module response
** Returned value:		TRUE is successful
**
*****************************************************************************/
uint32_t wiFlyPing(char *host, uint32_t count, char *response)
{
	uint32_t ret = FALSE;
	if (uart < MAX_UART_NUMBER && xMutex != NULL)
	{
		xSemaphoreTake( xMutex, portMAX_DELAY );
		{
			count = count %99;
			uartPrintfLn(uart, "ping %s %i", host, count-1);
			response[0] = '\0';
			static char strCount[7];
			sprintf(strCount, "seq=%i", count);
			if(uartResponse(uart, strCount, RESPONSE_TIME_OUT, string, RESPONSE_BUFF_LEN) == TRUE)
			{
				sprintf(response, "Ping Ok\r\n");
				char *search = "\r\n";
				char *token;
				token = strtok((char*)string, search);
				while(token != NULL)
				{
					if(strstr(token, "seq=") != NULL)
					{
						sprintf(response,"%s%s\r\n", response, token);
					}
					token = strtok(NULL, search);
				}
			}
			ret = TRUE;
		}
		xSemaphoreGive( xMutex );
	}
	return ret;
}

/*****************************************************************************
** Function name:		wiFlyConnect
**
** Descriptions:		Configure the module to make connections, associating
** 						with a access point (AP)
**
** parameters:			enum for the selected security mode
** 						char* ssid,the wlan to associate with. 1-32 chars
** 						char * phrase, the passphrase for WPA and WPA2 security
** 						char* for the return message of the module response
** Returned value:		TRUE is successful
**
*****************************************************************************/
uint32_t wiFlyConnect(enumSecurity mode, char* ssid, char* phrase, char* response)
{
	uint32_t ret = FALSE;
	if (uart < MAX_UART_NUMBER)
	{
		uint32_t iMode =0;
		switch(mode)
		{
			case OPEN:
				iMode =0;
				break;
			case WEP128:
				iMode =1;
				break;
			case WAP1:
				iMode =2;
				break;
			case MIXED:
				iMode =3;
				break;
			case WPA2PSK:
				iMode =4;
				break;
			case ADHOC:
				iMode =6;
				break;
			case WEP64:
				iMode =8;
				break;
		}

		uartPrintfLn(uart, "set wlan auth %i", iMode);
		response[0] = '\0';
		checkResponse(uart, "AOK", "Set wlan auth Ok\r\n", response);

		uartPrintfLn(uart, "set wlan ssid %s", ssid);
		checkResponse(uart, "AOK", "Set wlan ssid Ok\r\n", response);

		uartPrintfLn(uart, "set wlan phrase %s", phrase);
		checkResponse(uart, "AOK", "Set wlan phrase Ok\r\n", response);

		uartPrintfLn(uart, "join %s", ssid);
		checkResponse(uart, "Associated!", "Begin join Ok\r\n", response);
		ret = checkResponse(uart, "GW=", "End join Ok", response);
	}
	return ret;
}

/*****************************************************************************
** Function name:		wiFlyDisconnect
**
** Descriptions:		disconnects the module from the currently 
**						associated Access Point.
**
** parameters:			char* for the return message of the module response
** Returned value:		TRUE is successful
**
*****************************************************************************/
uint32_t wiFlyDisconnect(char* response)
{
	uint32_t ret = FALSE;
	if (uart < MAX_UART_NUMBER)
	{
		uartPrintfLn(uart, "leave");
		response[0] = '\0';
		ret = checkResponse(uart, ">", "Leave Ok", response);
	}
	return ret;
}

/*****************************************************************************
** Function name:		wiFlyGetChannel
**
** Descriptions:		Get the wlan channel number
**
** parameters:			char* for the return message of the module response
** Returned value:		TRUE is successful
**
*****************************************************************************/
uint32_t wiFlyGetChannel(char *response)
{
	uint32_t chan = 999999;
	if (uart < MAX_UART_NUMBER)
	{
		uartPrintfLn(uart, "get wlan");
		response[0] = '\0';
		if(uartResponse(uart, "Chan=", RESPONSE_TIME_OUT, string, RESPONSE_BUFF_LEN) == TRUE)
		{
			sprintf(response, "Get wlan Ok\r\n");
			char *search = "\r\n";
			char *token;
			token = strtok((char*)string, search);

			while(token != NULL)
			{
				if(strstr(token, "Chan=") != NULL)
				{
					char value[10];
					sscanf(token, "Chan=%99[^\r\n]", value);
					chan = atoi(value);
				}
				token = strtok(NULL, search);
			}
		}
	}
	return chan;
}

/*****************************************************************************
** Function name:		wiFlyWeb
**
** Descriptions:		Setup built in HTML client for getting or posting
** 						data to a web server.
**
** parameters:			char* URL of the destination web server
** 						char* for the return message of the module response
** Returned value:		TRUE is successful
**
*****************************************************************************/
uint32_t wiFlyWeb(char* url, char* response)
{
	uint32_t ret = FALSE;
	if (uart < MAX_UART_NUMBER)
	{
		static char host[100];
		static char get[150];

		sscanf(url, "http://%99[^/]%99[^\n]", host, get);

		uartPrintfLn(uart, "set dns name %s", host);
		checkResponse(uart, "AOK", "Set dns name Ok\r\n", response);

		uartPrintfLn(uart, "set com remote GET$%s", get);
		checkResponse(uart, "AOK", "Set com remote GET$ Ok\r\n", response);

		uartPrintfLn(uart, "set option format 1");
		checkResponse(uart, "AOK", "Set option format 1 Ok\r\n", response);

		uartPrintfLn(uart, "set ip proto 18");
		checkResponse(uart, "AOK", "Set ip proto 18 Ok\r\n", response);

		uartPrintfLn(uart, "set ip host 0");
		checkResponse(uart, "AOK", "Set ip host 0 Ok\r\n", response);

		uartPrintfLn(uart, "set ip remote 80");
		ret = checkResponse(uart, "AOK", "Set ip remote 80 Ok", response);
	}
	return ret;
}

/*****************************************************************************
** Function name:		wiFlyWebSend
**
** Descriptions:		Instructs the HTML client for posting data to a
** 						web server with the GET method.
**
** parameters:			char* URL for the web server
** 						char* for the return message of the module response
** Returned value:		TRUE is successful
**
*****************************************************************************/
uint32_t wiFlyWebSend(char* url, char* response)
{
	uint32_t ret = FALSE;
	if (uart < MAX_UART_NUMBER)
	{
		static char host[100];
	    static char get[150];
		response[0] = '\0';
		wiFlyWeb(url, response);
		sscanf(url, "http://%99[^/]%99[^\n]", host, get);
		uartPrintf(uart, "open %s 80\r\n", host);
		ret = checkResponse(uart, "*CLOS*", "\r\nOpen Ok", response);

		static char responseCmd[100];
		wiFlyOpen(responseCmd);
	}
	return ret;
}

/*****************************************************************************
** Function name:		wiFlyWebGet
**
** Descriptions:		Instructs the HTML client for getting an a integer data
** 						value from a web server
**
** parameters:			char* URL for the web server
** 						char* for the return message of the module response
** Returned value:		An a integer value returned from the server
**
*****************************************************************************/
uint32_t wiFlyWebGet(char * url, char* response)
{
	uint32_t num = -1;

	if (uart < MAX_UART_NUMBER)
	{
	   	static char host[100];
		static char get[150];
		response[0] = '\0';
		wiFlyWeb(url, response);

		sscanf(url, "http://%99[^/]%99[^\n]", host, get);
		uartPrintfLn(uart, "open %s 80", host);
		if(uartResponse(uart, "*CLOS*", RESPONSE_TIME_OUT, string, RESPONSE_BUFF_LEN) == TRUE)
		{
			//sprintf(response, "%s\r\nHttp Ok: %s", response, string);
			sprintf(response, "%s\r\nHttp Ok", response);
			static char *search = "\r\n";
			char *token;
			token = strtok((char*)string, search);
			while(token != NULL)
			{
				if(strstr(token, "*CLOS*") != NULL)
				{
					char value[10];
					sscanf(token, "%99[^*]*CLOS*[^\r\n]", value);
					num = atoi(value);
				}
				token = strtok(NULL, search);
			}
		}

		static char responseCmd[100];
		wiFlyOpen(responseCmd);
	}
	return num;
}

/*****************************************************************************
** Function name:		wiFlyTelnetServer
**
** Descriptions:		A queue based Telnet server for commands over WiFly
** 						connection.
** 						The command lines CRLF ended, received from the
** 						telnet client are send to the xQueueReceiveWifly queue.
**						The responses received from the xQueueSendWifly queue
**						are send to the telnet client.
**
** parameters:			xQueueHandle* for telnet client input commands
** 						xQueueHandle* for responses to the telnet client
** 						char* for the return message of the module response
** Returned value:		TRUE is successful
**
*****************************************************************************/
uint32_t wiFlyTelnetServer(xQueueHandle* xQueueReceiveWifly,
		xQueueHandle* xQueueSendWifly, char* response)
{
	portBASE_TYPE xStatus = pdPASS;
	static xMessage xStructToReceive = { 0, "" };
	static xMessage xStructToSend = { 0, "" };
	const uint32_t TimeOut =100;
	static uint8_t telnetMsg[512];
	uint32_t ret = FALSE;
	uint32_t exit = FALSE;

	if (uart < MAX_UART_NUMBER)
	{
		response[0] = '\0';

		uartPrintfLn(uart, "set com remote *Ready*");
		checkResponse(uart, "AOK", "Set com remote Ok\r\n", response);

		uartPrintfLn(uart, "set option format 0");
		checkResponse(uart, "AOK", "Set option format 0 Ok\r\n", response);

		uartPrintfLn(uart, "set ip proto 2");
		checkResponse(uart, "AOK", "Set ip proto 2 Ok\r\n", response);

		uartPrintfLn(uart, "set ip host 0");
		checkResponse(uart, "AOK", "Set ip host 0 Ok\r\n", response);

		uartPrintfLn(uart, "set ip remote 80");
		ret = checkResponse(uart, "AOK", "Set ip remote 80 Ok", response);

		while (!exit)
		{
			const portTickType xTicksToWait = 100 / portTICK_RATE_MS;
			if(xQueueReceive( xQueueSendWifly, &xStructToSend, xTicksToWait == pdPASS ))
			{
				// Data was successfully received from the queue, print out the received value.
				uartPrintfLn(uart, xStructToSend.message);
			}

			UartScanf(telnetMsg, uart, TimeOut);
			if(strlen((char*)telnetMsg)>0 && strstr((char*)telnetMsg, "\r\n"))
			{
				char *search = "\r\n";
				char *token;
				token = strtok((char*)telnetMsg, search);

				while(token != NULL)
				{
					if(strstr(token, "$$exit") != NULL){
						exit = TRUE;
						token = strtok(NULL, search);
					}
					else
					{
						if(strlen(token)<sizeof(xStructToReceive.message))
						{
							xStructToReceive.messageLength = strlen(token);
							strcpy(xStructToReceive.message, token);
							do{
								xStatus = xQueueSendToBack( xQueueReceiveWifly, ( void * ) &xStructToReceive, 0);
							} while  (xStatus != pdPASS );
						}
						token = strtok(NULL, search);
					}
				}
				telnetMsg[0] =0;
			}
			taskYIELD(); 		// Allow the other sender task to execute.
		}
	}
	return ret;
}

/*****************************************************************************
** Function name:		wiFlyVersion
**
** Descriptions:		return the software release version
**
** parameters:			char* for the return message of the module response
** Returned value:		Number of version
**
*****************************************************************************/
uint32_t wiFlyVersion(char* response)
{
	uint32_t ret = FALSE;
	if (uart < MAX_UART_NUMBER)
	{
		uartPrintfLn(uart, "ver");
		response[0] = '\0';
		if(uartResponse(uart, ">", RESPONSE_TIME_OUT, string, RESPONSE_BUFF_LEN) == TRUE)
		{
			const char *search = "\r\n";
			char *token;
			token = strtok((char*)string, search);
			while(token != NULL)
			{
				if(strstr(token, "WiFly Ver") != NULL)
				{
					char value[10];
					sscanf(token, "WiFly Ver %99[^,] ", value);
					sprintf(response, value);
					ret = TRUE;
				}
				token = strtok(NULL, search);
			}
		}
	}
	return ret;
}

/*****************************************************************************
** Function name:		wiFlyUpTime
**
** Descriptions:		Displays number of seconds since last powerup or reboot
**
** parameters:			char* for the return message of the module response
** Returned value:		Integer, the number of seconds
**
*****************************************************************************/
uint32_t wiFlyUpTime(char* response)
{
	uint32_t upTime =0;
	if (uart < MAX_UART_NUMBER)
	{
		uartPrintfLn(uart, "show time");
		response[0] = '\0';
		if(uartResponse(uart, "UpTime=", RESPONSE_TIME_OUT, string, RESPONSE_BUFF_LEN) == TRUE)
		{
			sprintf(response, "UpTime Ok");
			static char *search = "\r\n";
			char *token;
			token = strtok((char*)string, search);
			while(token != NULL)
			{
				if(strstr(token, "UpTime=") != NULL)
				{
					char value[10];
					sscanf(token, "UpTime=%99[^s]s[^\r\n]", value);
					upTime = atoi(value);
				}
				token = strtok(NULL, search);
			}
		}
	}
	return upTime;
}

/*****************************************************************************
** Function name:		wiFlyRssi
**
** Descriptions:		Displays current last received signal strength.
**
** parameters:			char* for the return message of the module response
** Returned value:		Integer, the signal level in dBm.
**
*****************************************************************************/
uint32_t wiFlyRssi(char* response)
{
	uint32_t rssi =0;
	if (uart < MAX_UART_NUMBER)
	{
		uartPrintfLn(uart, "show rssi");
		response[0] = '\0';
		if(uartResponse(uart, "RSSI=", RESPONSE_TIME_OUT, string, RESPONSE_BUFF_LEN) == TRUE)
		{
			sprintf(response, "Rssi Ok");
			static char *search = "\r\n";
			char *token;
			token = strtok((char*)string, search);
			while(token != NULL)
			{
				if(strstr(token, "RSSI=") != NULL)
				{
					char value[10];
					sscanf(token, "RSSI=(%99[^)]) dBm[^\r\n]", value);
					rssi = atoi(value);
				}
				token = strtok(NULL, search);
			}
		}
	}
	return rssi;
}

/*****************************************************************************
** Function name:		wiFlyGetTransmitPow
**
** Descriptions:		Gets the Wi-Fi transmit power.
** 						Returns a value from 1-12 corresponding to 1dBm to 12dBm.
** 						Value 0 is corresponding to 12dBm (max TX power).
**
** parameters:			char* for the return message of the module response
** Returned value:		dBm value.
**
*****************************************************************************/
uint32_t wiFlyGetTransmitPow(char* response)
{
	uint32_t pow = 999999;
	if (uart < MAX_UART_NUMBER)
	{
		uartPrintfLn(uart, "get wlan");
		response[0] = '\0';
		if(uartResponse(uart, "TxPower", RESPONSE_TIME_OUT, string, RESPONSE_BUFF_LEN) == TRUE)
		{
			sprintf(response, "Get wlan Ok\r\n");
			char *search = "\r\n";
			char *token;
			token = strtok((char*)string, search);

			while(token != NULL)
			{
				if(strstr(token, "TxPower=") != NULL)
				{
					char value[10];
					sscanf(token, "TxPower=%99[^\r\n]", value);
					pow = atoi(value);
				}
				token = strtok(NULL, search);
			}
		}
	}
	return pow;
}

/*****************************************************************************
** Function name:		wiFlySetTransmitPow
**
** Descriptions:		Sets the Wi-Fi transmit power. Accepts value from 1-12
** 						corresponding to 1dBm to 12dBm.
** 						Default is 0 corresponding to 12dBm (max TX power).
**
** parameters:			integer dBm value
** 						char* for the return message of the module response
** Returned value:		TRUE is successful
**
*****************************************************************************/
uint32_t wiFlySetTransmitPow(uint32_t pow, char* response)
{
	uint32_t ret = FALSE;
	if (uart < MAX_UART_NUMBER)
	{
		if (pow <=12)
		{
			uartPrintfLn(uart, "set wlan tx %i", pow);
			response[0] = '\0';
			ret = checkResponse(uart, "AOK", "Set wlan tx Ok", response);
		}
	}
	return ret;
}


/*****************************************************************************
** Function name:		wiFlySetCommIdle
**
** Descriptions:		sets the Idle Timer Value. This is the number of
** 						seconds with no transmit or receive data over TCP
** 						before the connection is closed automatically.
**						Default is 0, never disconnect on idle.
**
** parameters:			Seconds for the idle timer, debug message
** Returned value:		TRUE if response is Ok.
**
*****************************************************************************/
uint32_t wiFlySetCommIdle(uint32_t seconds, char* response)
{
	uint32_t ret = FALSE;
	if (uart < MAX_UART_NUMBER && xMutex != NULL)
	{
		xSemaphoreTake( xMutex, portMAX_DELAY );
		{

			uartPrintfLn(uart, "set comm idle %i", seconds);
			response[0] = '\0';
			ret = checkResponse(uart, "AOK", "Set comm idle Ok", response);

			if(ret == TRUE)
			{
				uartPrintfLn(uart, "save");
			 	ret = checkResponse(uart, "Storing in config", "Save Ok\r\n", response);
			}
		}
		xSemaphoreGive( xMutex );
	}
	return ret;
}

/*****************************************************************************
** Function name:		wiFlyBaudRate
**
** Descriptions:		set the UART baud rate. Valid settings are {2400, 4800,
** 						9600, 19200,38400, 57600, 115200, 230400, 460800,
** 						921600}.
**						The interface on the RN-134 does not work above 230400
**
**
** parameters:			Integer baud rate value
** 						char* for the return message of the module response
** Returned value:		TRUE is successful
**
*****************************************************************************/
uint32_t wiFlyBaudRate(uint32_t baud, char* response)
{
	uint32_t ret = FALSE;
	if (uart < MAX_UART_NUMBER)
	{
		const portTickType xDelay250ms = 250 / portTICK_RATE_MS;
		uartPrintfLn(uart, "set uart baud %i", baud);
		response[0] = '\0';
		ret = checkResponse(uart, "AOK", "Set uart baud Ok\r\n", response);

		uartPrintfLn(uart, "save");
		checkResponse(uart, "Storing in config", "Save Ok\r\n", response);

		uartPrintfLn(uart, "reboot");
		vTaskDelay(xDelay250ms);
		UARTInit(uart, baud);  // WiFly
		checkResponse(uart, "*Reboot*", "Reboot Ok\r\n", response);

		uartPrintf(uart, "$$$");
		vTaskDelay(xDelay250ms);
		checkResponse(uart, "CMD\r\n", "cmd Ok", response);
	}
	return ret;
}
